# This is a script that supposed to install MFT. 
#!/bin/bash

# Initiate global variables.
# pointing to the dir where this install script resides.
g_package_orig_dir=`cd ${0%*/*};pwd`
g_install_kernel=1
g_install_user=1
g_tmp_dir=/tmp/mft.$$.logs
g_machine_name=`uname -m`
g_only_for_source=0
g_old_mft_package="mft"
g_new_ofed_installed=""

g_mft_user="mft"
g_mft_kernel="kernel-mft"
g_mft_version="2.7.1"
g_mft_rel="7"
g_ofed14_suffix="ofed14"
g_prefix=""



# TODO: Add mft internal package uninstall
# TODO: How should I behave when an command fails.
# TODO: Add a cleanup option.

# The functions implementations

function echo_info () {
    echo "-I- $1"
}
function echo_warning () {
    echo "-W- $1"
}
function echo_debug () {
    echo "-D- $1" &> /dev/null
}
function echo_error () {
    echo "-E- $1"
}

function safe_rm_rf() {
    dir=$1
    case ${dir//\//} in
        usr|usrlocal|opt|bin|etc|lib|lib64|local|misc|sbin|var|boot|root|""|tmp)
        echo_error "Tried to remove directory $dir. Deletion aborted. Make sure the right destination paths were given"
        return 1
    esac
    /bin/rm -rf ${dir}
    return 0
}
function cleanup_old_mft () {
    echo_info "Removing any old MFT file if exists..."
    ${g_package_orig_dir}/old-mft-uninstall.sh --force >& /dev/null
}

function rpm_is_installed () {
    package_name=$1
    rpm -q ${package_name} &> /dev/null
    return $?
}

function check_rpms_present() {
    while [ "$1" ]; do
        package_name=$1
        rpm_is_installed ${package_name}; RC=$?
        if [ "${RC}" == "0" ]; then
            echo_warning "Failed to remove the package \"${package_name}\", try to remove it manually."
        fi          
        shift
    done     
}

function cleanup_rpms () {
    existing_packages=""
    # Preapre a list of the existing packages
    while [ "$1" ]; do
        package_name=$1
        rpm_is_installed ${package_name}; RC=$?
        if [ "${RC}" == "0" ]; then
            existing_packages="${existing_packages} ${package_name}"
        fi          
        shift
    done     
    # Uninstall the packages
    if [ "${existing_packages}" != "" ]; then
        echo_info "Removing the packages: $existing_packages..."
        rpm -e --allmatches ${existing_packages} &> /dev/null;
        rpm -e --allmatches --nodeps --noscripts ${existing_packages} &> /dev/null;
        # Check if the uninstallation operation passed
        check_rpms_present ${existing_packages}   
    fi
}

function cleanup_new_mft () {
    # Prepare the MFT package list.
    rpm_packages_list=""    
    if [ "${g_mft_user}" != "${g_old_mft_package}" ]; then 
        rpm_packages_list="$rpm_packages_list ${g_old_mft_package}"
    fi
    if [ "$g_install_user" == "1" ]; then 
        rpm_packages_list="$rpm_packages_list ${g_mft_user}"
    fi
    if [ "$g_install_kernel" == "1" ]; then 
        rpm_packages_list="$rpm_packages_list ${g_mft_kernel}"
    fi
    cleanup_rpms ${rpm_packages_list}
}


function cleanup_previous_mft_installation() {
    # Remove existing MFT temp directory
    safe_rm_rf ${g_tmp_dir}
    # Cleanup the new MFT RPMS if exist
    cleanup_new_mft  
    # Clean any old MFT installation which may have been installed without using the RPMs
    cleanup_old_mft
}

function check_arg() {
    if [ -z $2 ] || [[ "$2" == -* ]]; then
        echo "-E- Missing parameter after \"$1\" switch."
    exit 1
    fi
}

g_pref_op="--prefix"
g_help_op="--help"
g_tmp_dir_op="--tmpdir"
g_ofed14_compat_op="--ofed14-compat"
g_no_kernel_op="--without-kernel"

function print_help () {
     cat<<EOF
Usage: install.sh [${g_pref_op} <install-dir>][${g_help_op}] [${g_ofed14_compat_op} <yes|no>] 
                  [${g_no_kernel_op}]
Options:
    ${g_pref_op} <dir>           : Place bin/lib/include into <dir>.
    ${g_help_op}                   : Print this message.
    ${g_ofed14_compat_op} <yes|no> : Determines whether to install ofed14 compatibility MFT package or not. 
    ${g_no_kernel_op}         : Do not build kernel modules. PCI devices access will not be enabled.
EOF
   
}

function get_ofed14_arg () {
    arg=$1
    if [ "${arg}" == "yes" ]; then
        g_new_ofed_installed=0
    elif [ "${arg}" == "no" ]; then
        g_new_ofed_installed=1
    else 
        echo_error "${g_ofed14_compat_op} requires  'yes' or 'no' arguments, \"${arg}\" is an illegal argument"
        exit 1
    fi   
}

# get_user_options: Get the user option and updates the global variable to let the 
#                   isntall function install the needed RPMs.
function get_user_options () {
    while [ "$1" ]; do
        case $1 in
            "${g_help_op}")
                print_help
                exit 0
                ;;
            "${g_pref_op}")
                check_arg $1 $2
                g_prefix=$2
                shift
                ;;
            "${g_tmp_dir_op}")
                check_arg $1 $2
                g_tmp_dir=$2
                shift
                ;;
            "--only_for_source")
                g_only_for_source=1
                ;;
            "${g_no_kernel_op}")
                g_install_kernel=0
                ;;
            "${g_ofed14_compat_op}")
                check_arg $1 $2
                get_ofed14_arg $2
                shift
                ;;

            *)
            echo "-E- Bad switch \"$1\""
            print_help
            exit 1
        esac 
        
        shift
    done     
}

install_rpm() {
    package=$1
    tmp_dir=$2
    install_flags=$3
    
    file_name=${package##*/}
    log_file=${tmp_dir}/install_${file_name}.log
    rpm -Uvh --verbose ${install_flags} ${package} 2> ${log_file} ;   rc=$?
    if [ "$rc" != "0" ]; then 
        echo_error "Failed to install the package \"${package}\"', for more details see ${log_file}"
        exit 1
    fi      
}

function install_kernel_rpm() {
    kernel_src_rpm=$1
    kernel_rpm_dir=${g_tmp_dir}/topdir
    rpm_dir=${kernel_rpm_dir}/RPMS/*
    build_log="${g_tmp_dir}/rpm_build.log"

    echo_debug "Kernel source RPM package is: ${kernel_src_rpm}"

    mkdir -p ${kernel_rpm_dir}/{SPECS,SOURCES,RPMS,BUILD,SRPMS}
    # cp ${kernel_src_rpm}  ${srpm_dir}
    
    echo_info "Building the MFT kernel binary RPM..."    
    rpmbuild --define "_topdir ${kernel_rpm_dir}" --rebuild ${kernel_src_rpm} >& ${build_log}; rc=$?
    if [ "$rc" !=  "0" ]; then
        echo_error   "Failed to build the kernel MFT binary RPM from the sources RPM. For more details, see ${build_log}"
        exit 1
    fi
    
    cd ${rpm_dir}
    # TODO: Move this code to another function
    echo_info "Installing the MFT RPMs..."
    
    kernel_rpm=`ls ${g_mft_kernel}-${g_mft_version}* | grep -v "debug-info"`
    install_rpm ${kernel_rpm} ${g_tmp_dir}
    cd - &> /dev/null
}
# Install_rpms: Intsall the needed RPMs according to the user options and machine info.
function Install_rpms () {   
    rpm_dir=${g_package_orig_dir}/RPMS
    srpm_dir=${g_package_orig_dir}/SRPMS
    
    # Update the rpm packages path
    if [ "${g_new_ofed_installed}" == "1" ]; then
        user_rpm=${rpm_dir}/${g_mft_user}-${g_mft_version}-${g_mft_rel}.${g_machine_name}.rpm
    else 
        user_rpm=${rpm_dir}/${g_mft_user}-${g_mft_version}-${g_mft_rel}_${g_ofed14_suffix}.${g_machine_name}.rpm
    fi    
    
    kernel_src_rpm=${srpm_dir}/${g_mft_kernel}-${g_mft_version}-${g_mft_rel}.src.rpm

    # Install kernel RPM.
    if [ "$g_install_kernel" == "1" ]; then 
        install_kernel_rpm ${kernel_src_rpm}
        
    fi
    
    # Install user RPM.
    if [ "$g_install_user" == "1" ]; then
        if [ "${g_prefix}" != "" ]; then
            export MFT_NEW_PREFIX_ENV=${g_prefix}
            install_rpm ${user_rpm} ${g_tmp_dir} "--relocate /usr=${g_prefix}"
        else
            unset MFT_NEW_PREFIX_ENV
            install_rpm ${user_rpm} ${g_tmp_dir}
        fi
    fi
}

function cleanup_residues() {
    safe_rm_rf ${g_tmp_dir}
}
function update_machine_name() {
    if [ "$g_machine_name" == "i586" ] || [ "$g_machine_name" == "i386" ] || [ "$g_machine_name" == "i486" ] || [ "$g_machine_name" == "i686" ]; then
            g_machine_name="x86"
    fi
}

function installation_init () {
    mkdir -p ${g_tmp_dir}
    #update_machine_name
}

# version_triplet_less_than gets 2 params (version1 and version2) and behaves as follow:
# version1 < version2 - retrun 0
# version1 >= version2 - return 1
function version_triplet_less_than() {
    version1=$1
    version2=$2
    for i in `seq 1 3`; do
        ver1=`echo $version1 | cut -d"." -f $i`
        ver2=`echo $version2 | cut -d"." -f $i`
        if [ ${ver1} -lt ${ver2} ]; then
            return 0
        elif [ ${ver1} -gt ${ver2} ]; then
            return 1
        fi
    done
    return 1
}


function check_libibmad_rpm_version() {
    ibmad_version=$1
    min_new_version="1.3.4"
    version_triplet_less_than ${ibmad_version} ${min_new_version}; g_new_ofed_installed=$?
}

g_libibmad_name="libibmad"
function find_lib() {
    lib_name=$1
    
    if [ ${g_machine_name} == "x86" ] || [ ${g_machine_name} == "ia64" ]; then
        lib="lib"
    else 
        lib="lib64"
    fi
    
    pathes="/${lib} /usr/${lib} /usr/local/${lib}"
    
    for lib_path in ${pathes}; do
        if [ -f ${lib_path}/${lib_name} ]; then 
            # The given lib found
            return 0
        fi    
    done
    # The given lib was not found
    return 1
}


function check_libibmad_lib_version() {
    find_lib ${g_libibmad_name}.so.5; rc=$?
    if [ "$rc" == "0" ]; then 
        # libibmad.s0.5 found, installing 'new'
        g_new_ofed_installed=1
    else 
        find_lib ${g_libibmad_name}.so.1; rc1=$? 
        find_lib ${g_libibmad_name}.so.2; rc2=$? 
        if [ "$rc1" == "0" ] || [ "$rc2" == "0" ]; then 
            # old libibmad found, installing 'ofed_14' package
            g_new_ofed_installed=0
        else 
            # No libibmad found installing 'new' RPM.
            g_new_ofed_installed=1
        fi
    fi
}

function check_libibmad_version() {
    ibmad_version=`rpm -qa ${g_libibmad_name} | cut -d"-" -f 2`   
    regexp=[0-9]\.[0-9]\.[0-9]
    if [[ "$ibmad_version" =~ ${regexp} ]]; then 
        # There is a libibmad RPM installed, all you need now is to check if its version is old or new.
        check_libibmad_rpm_version ${ibmad_version}
    else 
        # No rpm found, checking if we have libibmad which was installed manually.
        check_libibmad_lib_version
    fi
}    

# Only root can install this package.
if [ $UID != 0 ]; then
    echo_error "You must be root to install MFT"
    exit 1
fi

# Get user options.
get_user_options $@ 

# Update the machine name
update_machine_name

# Check which libibmad is installed on this machine to determine which rom should be installed.
if [ "${g_new_ofed_installed}" == "" ]; then 
    check_libibmad_version
fi    

if [ ${g_only_for_source} == "1" ]; then 
    return 0
fi

# Cleanup existing MFT installation (Old or new) and old istallation tmp_dir
cleanup_previous_mft_installation

# Create needed folders or init
installation_init


# Execute special options

# Install the RPMS according to the machine setup and the user given flags.
Install_rpms

# Cleaup this build residues
cleanup_residues
